@@ -3,7 +3,7 @@ from .hash_object import smart_hash
 
 
 class Cache:
-	def __init__(self, path, spark=None):
+	def __init__(self, path, spark=None, echo=1):
 		"""
 		:type path: amazonian.S3.S3Path or Path or str
 		:type spark: pyspark.sql.session.SparkSession
@@ -14,6 +14,7 @@ class Cache:
 		self._path = path
 		self._spark = spark
 		self._items = set()
+		self._echo = echo
 
 	@staticmethod
 	def get_key(item):
@@ -30,13 +31,32 @@ class Cache:
 		key = self.get_key(item)
 		path = self.get_path(key)
 		result = path.save(obj=value, mode='overwrite')
+		if self._echo:
+			print(f'{result} saved!')
 		self._items.add(key)
 		return result
 
 	def get_item(self, item):
 		key = self.get_key(item)
 		path = self.get_path(key)
-		return path.load(spark=self._spark)
+		if path.exists():
+			if self._echo:
+				print(f'{path.path} loaded!')
+			return path.load(spark=self._spark)
+		else:
+			pickle_path = path + '.pickle'
+			if pickle_path.exists():
+				if self._echo:
+					print(f'{pickle_path.path} loaded!')
+				return pickle_path.load()
+			else:
+				parquet_path = path + '.parquet'
+				if parquet_path.exists():
+					if self._echo:
+						print(f'{parquet_path.path} loaded!')
+					return parquet_path.load(spark=self._spark)
+				else:
+					raise KeyError(f'key {key} does not exist!')
 
 	def contains(self, item):
 		key = self.get_key(item)
@@ -47,6 +67,10 @@ class Cache:
 			if path.exists():
 				self._items.add(key)
 				return True
+			elif (path + '.pickle').exists():
+				return True
+			elif (path + '.parquet').exists():
+				return True
 			else:
 				return False
 
